@@ -11,13 +11,14 @@ use warnings;
 
 use lib './lib';
 use Net::Proxmox::VE;
+use IO::Socket::SSL qw(SSL_VERIFY_NONE);
 use Data::Dumper;
 use Getopt::Long;
 
 my $host     = 'host';
 my $username = 'user';
 my $password = 'pass';
-my $debug    =  undef;
+my $debug    = undef;
 my $realm    = 'pve'; # 'pve' or 'pam'
 
 GetOptions (
@@ -25,7 +26,7 @@ GetOptions (
     'username=s' => \$username,
     'password=s' => \$password,
     'debug'      => \$debug,
-    'realm'      => \$realm,
+    'realm=s'    => \$realm,
 );
 
 my $pve = Net::Proxmox::VE->new(
@@ -34,6 +35,10 @@ my $pve = Net::Proxmox::VE->new(
     password => $password,
     debug    => $debug,
     realm    => $realm,
+    ssl_opts => {
+        SSL_verify_mode => SSL_VERIFY_NONE,
+        verify_hostname => 0
+    },
 );
 
 die "login failed\n"          unless $pve->login;
@@ -43,26 +48,13 @@ die "unsupport api version\n" unless $pve->api_version_check;
 # list openvz virtual machines by requesting
 my $resources = $pve->get("/cluster/resources");
 
-foreach my $item( @$resources ) {
+for my $item ( @$resources ) {
     next unless $item->{type} eq 'openvz';
 
-    print "id: " .        $item->{id} . "\n"; 
-    print "cpu: " .       $item->{cpu} . "\n";
-    print "disk: " .      $item->{disk} . "\n";
-    print "maxcpu: " .    $item->{maxcpu} . "\n";
-    print "maxdisk: " .   $item->{maxdisk} . "\n";
-    print "maxmem: " .    $item->{maxmem} . "\n";
-    print "mem: " .       $item->{mem} . "\n";
-    print "node: " .      $item->{node} . "\n";
-    print "type: " .      $item->{type} . "\n";
-    print "uptime: " .    $item->{uptime} . "\n";
-    print "diskread: " .  $item->{diskread} . "\n";
-    print "diskwrite: " . $item->{diskwrite} . "\n";
-    print "name: " .      $item->{name} . "\n";
-    print "netin: " .     $item->{netin} . "\n";
-    print "netout: " .    $item->{netout} . "\n";
-    print "status: " .    $item->{status} . "\n";
-    print "template: " .  $item->{template} . "\n";
-    print "vmid: " .      $item->{vmid} . "\n";
-    print "\n";
+    for my $k (sort keys %$item) {
+        print $k . ": " . $item->{$k} . "\n";
+    }
+
+    print "\n"
+
 }
